from cement.core import handler, hook
from cement.utils import test
from nepho import cli
from nepho.cli.base import Nepho
import nose


class NephoTestApp(Nepho):
    class Meta:
        argv = []
        config_files = []


# Test Parameter
class a_TestNephoParameter(test.CementTestCase):
    app_class = NephoTestApp

    def setUp(self):
        super(a_TestNephoParameter, self).setUp()

        self.reset_backend()

        app = self.make_app(argv=['cloudlet', '--quiet', 'uninstall', 'nepho-example'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['cloudlet', '--quiet', 'install', 'nepho-example'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['parameter', 'set', 'test_key', 'test_value'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_parameter_list(self):
        app = self.make_app(argv=['parameter', 'list'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_parameter_set(self):
        app = self.make_app(argv=['parameter', 'set', 'test_key', 'test_value'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_parameter_add(self):
        app = self.make_app(argv=['parameter', 'add', 'test_key', 'test_value'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_parameter_get(self):
        app = self.make_app(argv=['parameter', 'get', 'test_key'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_parameter_unset(self):
        app = self.make_app(argv=['parameter', 'set', 'test_key', 'test_value'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['parameter', 'unset', 'test_key', 'test_value'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_parameter_remove(self):
        app = self.make_app(argv=['parameter', 'set', 'test_key', 'test_value'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['parameter', 'remove', 'test_key', 'test_value'])
        app.setup()
        app.run()
        app.close()

    def test_nepho_parameter_delete(self):
        app = self.make_app(argv=['parameter', 'set', 'test_key', 'test_value'])
        app.setup()
        app.run()
        app.close()
        app = self.make_app(argv=['parameter', 'delete', 'test_key', 'test_value'])
        app.setup()
        app.run()
        app.close()



#!/usr/bin/env python
# coding: utf-8

import os
from copy import copy

from cement.core import controller, hook
from termcolor import colored
from nepho.cli import base


class NephoScopeController(base.NephoBaseController):
    class Meta:
        label = 'scope'
        interface = controller.IController
        stacked_on = 'base'
        stacked_type = 'nested'
        description = 'set a cloudlet (and optionally blueprint) scope for future commands'
        usage = "nepho scope <action> [cloudlet] [blueprint]"
        arguments = [
            (['cloudlet'], dict(help="Name of a cloudlet", nargs='?')),
            (['blueprint'], dict(help="Name of a blueprint (optional)", nargs='?')),
        ]

    @controller.expose(aliases=['set'], help="set a scope for future commands or view current scope")
    def default(self):
        new_cn = self.app.pargs.cloudlet
        new_bp = self.app.pargs.blueprint
        old_cn = self.app.config.get('scope', 'cloudlet')
        old_bp = self.app.config.get('scope', 'blueprint')

        self.app.log.debug('Setting scope to cloudlet: %s, blueprint: %s' % (new_cn, new_bp))

        if new_cn:
            self.app.config.set('scope', 'cloudlet', new_cn)
            if new_bp:
                # Only delete the blueprint if cloudlet is being explicitly set
                # (i.e. user isn't just viewing current scope printout)
                self.app.config.set('scope', 'blueprint', new_bp)
            else:
                self.app.config.set('scope', 'blueprint', '')

            if new_cn != old_cn or new_bp != old_bp:
                print "Set default command scope to " + colored(new_cn, "cyan") + " " + colored(new_bp or "", "yellow")
                save_config(self)
        elif old_cn is '':
            print "Default command scope is unset. Run nepho scope <cloudlet> [blueprint] to set."
        else:
            print_scope(self)

    @controller.expose(help="unset current scope", aliases=['clear'])
    def unset(self):
        self.app.log.debug('Unsetting scope.')
        self.app.config.set('scope', 'cloudlet', '')
        self.app.config.set('scope', 'blueprint', '')
        save_config(self)

        print "Default command scope is now unset. Run nepho scope <cloudlet> [blueprint] to set."


def print_scope(app_obj):
    if app_obj.app.config.get('scope', 'cloudlet') is not '':
        print base.DISP_DASH * 80
        print " Using default command scope " + colored(app_obj.app.config.get('scope', 'cloudlet'), "cyan") + " " + colored(app_obj.app.config.get('scope', 'blueprint') or "", "yellow")
        print base.DISP_DASH * 80
    else:
        pass


def save_config(app_obj):
    # Write out the working configuration to the lowest accessible config file
    # on the stack after stripping extraneuous values. This has gotten quite
    # convoluted and now I am regretting the whole exercise of using built-in
    # config handling vs rolling our own.
    config_files = app_obj.app._meta.config_files
    config_files.reverse()
    config_obj = copy(app_obj.app.config)
    for section in [section for section in config_obj.sections() if section.find('controller.') != -1]:
        config_obj.remove_section(section)
    config_obj.remove_section('log')
    for cf in config_files:
        if os.access(os.path.dirname(cf), os.W_OK | os.X_OK):
            try:
                with open(cf, 'wb') as configfile:
                    config_obj.write(configfile)
                break
            except:
                pass
        else:
            continue
    else:
        print "Unable to write Nepho configuration -- no writable config file location found."
        print "Possible locations are: %s" % (app_obj.app._meta.config_files)



# coding: utf-8
# flake8: noqa

import os
from os import path
import subprocess
import re
import yaml
import shutil
from termcolor import colored

import vagrant
import nepho
from nepho.core.common import cwd, execute

class VagrantProvider(nepho.core.provider.AbstractProvider):
    """An infrastructure provider class for Vagrant"""

    PROVIDER_ID = "vagrant"
    TEMPLATE_FILENAME = "Vagrantfile"

    def __init__(self, config, scenario=None):
        nepho.core.provider.AbstractProvider.__init__(self, config, scenario)
        self.params = {
            'VagrantBackend': 'virtualbox',
            'BoxName': None
        }

        if scenario is not None:
            if scenario.name is not None:
                self.stack_name = scenario.name
            else:
                context = scenario.context
                self.stack_name = "%s-%s" % (context['cloudlet']['name'],
                                             context['blueprint']['name'])
        else:
            print "scenario is none"

    @property
    def vagrantfile_path(self):
        # Lazy-load path because it requires a valid context.
        return os.path.join(
            self.scenario.context['cloudlet']['path'], 'resources',
            self.scenario.context['blueprint']['name'])

    @property
    def payload_path(self):
        return os.path.join(
            self.scenario.context['cloudlet']['path'], 'payload')

    @property
    def vagrant_backend(self):
        context = self.scenario.context
        if context['parameters']['VagrantBackend'] is None:
            print colored("Error: ", "red"), "Vagrant requires a valid backend provider, such as 'virtualbox'."
            print "Use \"nepho parameter set VagrantBackend <value>\" to set a parameter."
            exit(1)
        return context['parameters']['VagrantBackend']

    def validate_template(self, template_str):
        return "Validation:\n  Vagrant does not support validation."

    def deploy(self, app_obj):
        """Deploy a given pattern."""
        if app_obj.pargs.debug is True:
            print "Vagrant does not support debug logging"

        context = self.scenario.context
        raw_template = self.scenario.template

        if context['parameters']['BoxName'] is not None:
            name = context['parameters']['BoxName']
            url = context['parameters']['BoxUrl']

            if name not in self._list_box_names(self.vagrant_backend):
                try:
                    print " - Downloading box %s" % (name)
                    execute('vagrant box add %s %s' % (name, url))
                except subprocess.CalledProcessError:
                    print colored("Error: ", "red") + 'Attempt to download box exited with a non-zero exit code.'
                    exit(1)

        print " - Copying payload to working directory"

        working_path = self._working_path(app_obj)

        if os.path.isdir(working_path):
            print colored("Error: ", "red") + "Working directory already exists"
            print "Either a stack of this name is already running or an old working directory exists."
            exit(1)

        try:
            shutil.copytree(self.payload_path, os.path.join(working_path, 'payload'))
        except:
            print colored("Error: ", "red") + "Unable to create payload directory"
            shutil.rmtree(working_path)
            exit(1)

        print " - Generating Vagrantfile"
        try:
            f = open(os.path.join(working_path, 'Vagrantfile'), 'w')
            f.write(raw_template)
            f.close()
        except:
            print colored("Error: ", "red") + "Unable to generate Vagrantfile"
            exit(1)

        with cwd(working_path):
            v = vagrant.Vagrant()
            vm_name = None
            try:
                print " - Running Vagrant from: %s" % (working_path)
                v.up(provider=self.vagrant_backend, vm_name=vm_name)
                print 'Vagrant run complete. Run "nepho stack status" for details or "nepho stack access" to connect.'
            except subprocess.CalledProcessError:
                print colored("Error: ", "red") + 'Vagrant exited with a non-zero exit code, but your VM may be running.  Run "nepho stack status" for details.'

    def status(self, app_obj):
        working_path = self._working_path(app_obj)

        if not os.path.isdir(working_path):
            print colored("Error: ", "red") + "Working directory does not exist"
            exit(1)
        else:
            with cwd(working_path):
                v = vagrant.Vagrant()
                instances = v.status()

                print "Name:        %s" % self.stack_name
                print "Working Dir: %s" % working_path
                print
                print colored(" %-32s %-24s " % ("Instance", "Status"), 'white', 'on_blue')
                for k, v in instances.items():
                    print " %-32s %-24s " % (k, v)

                print "\nTo run Vagrant commands directly, change to the working directory listed above.\n"

    def access(self, app_obj):
        if not os.path.isdir(self._working_path(app_obj)):
            print colored("Error: ", "red") + "Working directory does not exist"
            exit(1)
        else:
            with cwd(self._working_path(app_obj)):
                v = vagrant.Vagrant()
                ssh_connect_string = v.user_hostname_port()
                vagrant_binary = vagrant.VAGRANT_EXE
                os.execlp(vagrant_binary, "", "ssh")

    def destroy(self, app_obj):
        if not os.path.isdir(self._working_path(app_obj)):
            print colored("Error: ", "red") + "Working directory does not exist"
            exit(1)
        else:
            with cwd(self._working_path(app_obj)):
                v = vagrant.Vagrant()
                print " - Terminating instance(s)"
                v.destroy()
                print " - Removing working directory"
                shutil.rmtree(self._working_path(app_obj))

    def _list_boxes(self):
        listing = subprocess.check_output('vagrant box list', shell=True)
        boxes = []
        for line in listing.splitlines():
            m = re.search(r'^\s*(?P<name>.+?)\s+\((?P<provider>[^)]+)\)\s*$', line)
            if m:
                boxes.append((m.group('name'), m.group('provider')))
        return boxes

    def _list_box_names(self, backend):
        return [box[0] for box in self._list_boxes() if box[1] == backend]

    def _working_path(self, app_obj):
        return os.path.join(app_obj.config.get('nepho', 'working_dir'), self.stack_name)




# coding: utf-8


class AbstractProvider:
    """An abstract infrastructure provider class."""

    def __init__(self, config, scenario=None):
        self.config = config
        self.scenario = scenario
        self.params = None

    def validate_template(self, template):
        pass

    def format_template(self, template):
        return template

    def deploy(self):
        pass

    def status(self):
        pass

    def access(self):
        pass

    def destroy(self):
        pass



import glob
import subprocess
import os
import sys


class cwd:
    """
    Context manager for changing the current working directory

    with cwd("foobar"):
        do a thing
    """
    def __init__(self, newPath):
        self.newPath = newPath

    def __enter__(self):
        self.savedPath = os.getcwd()
        os.chdir(self.newPath)

    def __exit__(self, etype, value, traceback):
        os.chdir(self.savedPath)


def execute(command):
    process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

    # Poll process for new output until finished
    while True:
        nextline = process.stdout.readline()
        if nextline == '' and process.poll() is not None:
            break
        sys.stdout.write(nextline)
        sys.stdout.flush()

    output = process.communicate()[0]
    exitCode = process.returncode

    if (exitCode == 0):
        return output
    else:
        raise subprocess.ProcessException(command, exitCode, output)


def merge(user, system):
    """
    Merge two dicts, user and system, recursively, with user taking precedence.
    """
    if isinstance(user, dict) and isinstance(system, dict):
        for k, v in system.iteritems():
            if k not in user:
                user[k] = v
            else:
                user[k] = merge(user[k], v)
    return user


def select_list(self, items_list=[], all=False, desc="Select an item:"):
    """
    Return the item selected by the user, or, optionally, "all".  If there is
    only one list item, return it without prompting.
    """
    if items_list == []:
        return
    elif len(items_list) == 1:
        return items_list[0]
    else:
        item_incr = 0
        print ""
        for one_item in items_list:
            item_incr += 1
            print "  %d) %s" % (item_incr, one_item.strip())
        if all is True:
            print "  0) All"
        user_item = int()
        try:
            user_item = input("\n%s [1]: " % (desc))
        except NameError:
            # User did not input a number
            user_item = -1
        except SyntaxError:
            # User accepted default
            user_item = 1

        if user_item == 0 and all is True:
            return items_list
        elif user_item <= item_incr and user_item > 0:
            return items_list[user_item - 1]
        else:
            print "Invalid selection, please select a number from the list."
            exit(1)


def all_cloudlets(self):
    # Collect the filesystem paths to every cloudlet into one list
    cloudlet_paths = list()
    for one_dir in self.cloudlet_dirs:
        cloudlet_paths.extend(glob.glob(os.path.join(one_dir, '*')))

    return cloudlet_paths


def find_cloudlet(self, name, multiple=False):
    cloudlet_paths = all_cloudlets(self)
    paths = [path for path in cloudlet_paths if name in path]
    if multiple is True:
        return paths
    else:
        return paths[0]


def all_blueprints(self, name):
    cloudlet = find_cloudlet(self, name)
    blueprint_files = list()
    if os.path.isdir(os.path.join(cloudlet, "blueprints")):
        blueprint_files.extend(glob.glob(os.path.join(cloudlet, "blueprints", '*.yaml')))
        return blueprint_files
    else:
        return None


def find_blueprint(self, cloudlet, name):
    blueprint_paths = all_blueprints(self, cloudlet)
    paths = [path for path in blueprint_paths if name in path]
    return paths[0]



# coding: utf-8

from nepho.core import common, resource, pattern, provider
from os import path


class AnisbleProvider:
    """An infrastructure provider class for ansible"""

    PROVIDER_ID = "ansible"
    TEMPLATE_FILENAME = "playbook.yaml"

    def deploy(self):
        """Deploy a given pattern."""
        pass

    def undeploy(self):
        pass



# coding: utf-8
#
# Parts shamelessly stolen from:
#
#   http://www.technobabelfish.com/2013/08/boto-and-cloudformation.html
#
#
import os
import sys
import time
import yaml
import json
import collections
import shutil
import tempfile
from termcolor import colored

import boto
import boto.cloudformation
import boto.s3.connection
import boto.iam.connection

from ast import literal_eval

from textwrap import TextWrapper

import signal
from cement.core import exc

import nepho


class AWSProvider(nepho.core.provider.AbstractProvider):
    """
    An infrastructure provider class for AWS CloudFormation

    Note: if you create an output named SSHEndpoint, then you can
      use the "nepho stack access" command to connect directly to the stack.

    """

    PROVIDER_ID = "aws"
    TEMPLATE_FILENAME = "cf.json"

    def __init__(self, config, scenario=None):
        nepho.core.provider.AbstractProvider.__init__(self, config, scenario)
        self.params = {
            'AWSAccessKeyID': None,
            'AWSSecretAccessKey': None,
            'AWSRegion': None,
            'KeyName': None,
        }
        self._connection = None
        self._s3_conn = None
        self._iam_conn = None

        if scenario is not None:
            if scenario.name is not None:
                self.stack_name = scenario.name
            else:
                context = scenario.context
                self.stack_name = "%s-%s" % (context['cloudlet']['name'],
                                             context['blueprint']['name'])
        else:
            print "scenario is none"

    @property
    def connection(self):
        """
        Lazy-load connection when needed. This allows for including the provider
        in context creation even though the provider connection requires a valid
        context.
        """
        if self._connection is None:
            (access_key, secret_key, region) = self._load_aws_connection_settings()
            self._connection = boto.cloudformation.connect_to_region(
                region, aws_access_key_id = access_key, aws_secret_access_key = secret_key)
            if self._connection is None:
                print "Boto connection to CloudFormation failed. Please check your credentials."
                exit(1)
        return self._connection

    @property
    def s3_conn(self):
        if self._s3_conn is None:
            (access_key, secret_key, region) = self._load_aws_connection_settings()
            self._s3_conn = boto.s3.connection.S3Connection(
                aws_access_key_id = access_key, aws_secret_access_key = secret_key)
            if self._s3_conn is None:
                print "Boto connection to S3 failed. Please check your credentials."
                exit(1)
        return self._s3_conn

    @property
    def iam_conn(self):
        if self._iam_conn is None:
            (access_key, secret_key, region) = self._load_aws_connection_settings()
            self._iam_conn = boto.iam.connection.IAMConnection(
                aws_access_key_id = access_key, aws_secret_access_key = secret_key)
            if self._iam_conn is None:
                print "Boto connection to IAM failed. Please check your credentials."
                exit(1)
        return self._iam_conn

    def validate_template(self, template_str):

        # Attempt to validate that template is valid JSON, and clean it up
        try:
            template_dict = json.loads(template_str)
            template_str  = json.dumps(template_dict, indent=2, separators=(',', ': '))
        except Exception as e:
            ret = "Validation:\n  Template JSON is not valid!\n"
            ret += "Error:\n  %s\n" % (e)
            return ret

        try:
            compact_template = json.dumps(json.loads(template_str), separators=(',', ':'))
            t = self.connection.validate_template(template_body=compact_template)

            ret = "Validation:\n  Template is valid\n"
            ret += "Description:\n %s\n" % t.description
            ret += "Parameters:"
            for p in t.template_parameters:
                ret += "\n  %s" % (p.parameter_key)
        except boto.exception.BotoServerError as e:
            ret = "Boto Server Error:\n  Template is not valid!\n"
            ret += "Error:\n  %s\n" % (e.error_code)
            ret += "Description:\n  %s" % (e.body)
        return ret

    def format_template(self, raw_template):
        """Pretty formats a CF template"""

        cf_dict = parse_cf_json(raw_template)
        return get_cf_json(cf_dict, pretty=True)

    def deploy(self, app_obj):
        """Deploy a given pattern."""
        if app_obj.pargs.debug is True:
            boto.set_stream_logger('aws_provider')

        context = self.scenario.context

        raw_template = self.scenario.template

        template_dict = parse_cf_json(raw_template)
        template_param_names = template_dict['Parameters'].keys()
        # Minimize JSON to save space
        template_json = json.dumps(template_dict, separators=(',', ':'))

        params = list()
        for item in context['parameters'].items():
            if item[0] in template_param_names:
                if item[1] is None:
                    print colored("Error: ", "red"), "Required parameter %s is not set" % item[0]
                    exit(1)
                else:
                    params.append(item)
            else:
                print colored("Warning: ", "yellow"), "Nepho parameter %s is not present in the CloudFormation template" % item[0]

        print " - Determining owner information"
        try:
            iam_user = self.iam_conn.get_user()
            if 'OwnerId' in template_param_names:
                params.append(('OwnerId', iam_user.user_id))
            if 'OwnerName' in template_param_names:
                params.append(('OwnerName', iam_user.user_name))
        except:
            print colored("Error: ", "red") + "Unable to get IAM username"
            exit(1)

        print " - Archiving payload for upload to S3"
        try:
            tmp_dir = tempfile.mkdtemp()
            payload = shutil.make_archive(
                os.path.join(tmp_dir, 'payload'), 'zip',
                os.path.join(context['cloudlet']['path'], 'payload'))
        except:
            print colored("Error: ", "red") + "Unable to create payload file"
            shutil.rmtree(tmp_dir)
            exit(1)

        # Upload payload to a private S3 bucket
        print " - Creating S3 bucket for payload"
        (access_key, secret_key, region) = self._load_aws_connection_settings()
        try:
            payload_bucket = self.s3_conn.create_bucket(
                'nepho-payloads-' + access_key.lower(), policy='private')

            payload_name = '%s/%s/payload.zip' % (context['cloudlet']['name'], context['blueprint']['name'])
            payload_key = boto.s3.key.Key(payload_bucket)
            payload_key.key = payload_name
        except Exception as e:
            print colored("Error: ", "red") + "Unable to create S3 bucket"
            print "(%s) %s " % (e.error_code, e.body)
            shutil.rmtree(tmp_dir)
            exit(1)

        print " - Uploading payload archive to S3"
        try:
            payload_key.set_contents_from_file(open(payload, 'rb'))
            params.append(('PayloadURL', payload_key.generate_url(3600)))
        except Exception as e:
            print colored("Error: ", "red") + "Unable to upload payload to S3"
            print "(%s) %s " % (e.error_code, e.body)
            exit(1)
        finally:
            shutil.rmtree(tmp_dir)

        try:
            print " - Creating CloudFormation stack"
            stack_id = self.connection.create_stack(
                self.stack_name,
                template_body = template_json,
                parameters = params,
                capabilities = ['CAPABILITY_IAM'],
                disable_rollback = True
            )

            try:
                self._show_status(self.stack_name)
            except exc.CaughtSignal:
                exit()
        except boto.exception.BotoServerError as e:
            print colored("Error: ", "red") + "Problem communicating with CloudFormation"
            # Use e.message instead of e.body as per: https://github.com/boto/boto/issues/1658
            msg = literal_eval(e.message)
            print "(%s) %s " % (msg["Error"]["Code"], msg["Error"]["Message"])
            exit(1)
        return stack_id

    def status(self, app_obj):
        """Check on the status of a stack within CloudFormation."""

        try:
            self._show_status(self.stack_name)
        except exc.CaughtSignal:
            exit()

    def access(self, app_obj):
        """Check on the status of a stack within CloudFormation."""

        # Return object of type boto.cloudformation.stack.Stack
        try:
            stack = self.connection.describe_stacks(stack_name_or_id=self.stack_name)

            # this will need to be improved ... basically a stub for now ...
            outputs = stack.outputs
            access_hostname = outputs['SSHEndpoint']
            return "ssh %s@%s" % ("ec2-user", access_hostname)
        except boto.exception.BotoServerError as be:
            # Actually ,this may just mean that there's no stack by that name ...
            print "Error communication with the CloudFormation service: %s" % (be)
            exit(1)

    def destroy(self, app_obj):
        """Delete a CloudFormation stack."""
        self.connection.delete_stack(stack_name_or_id=self.stack_name)

        try:
            self._show_status(self.stack_name)
        except exc.CaughtSignal:
            exit()

    def _load_aws_connection_settings(self):
        context = self.scenario.context
        required_params = ('AWSAccessKeyID', 'AWSSecretAccessKey', 'AWSRegion', 'KeyName')
        for param in required_params:
            if context['parameters'][param] is None:
                print colored("Error: ", "red"), "In order to use the AWS provider you must provide valid credentials."
                print "The following parameters are required:"
                print " ", "\n  ".join(required_params)
                print "Use \"nepho parameter set <key> <value>\" to set a parameter."
                exit(1)
        return (
            context['parameters']['AWSAccessKeyID'],
            context['parameters']['AWSSecretAccessKey'],
            context['parameters']['AWSRegion']
        )

    def _show_status(self, stack):
        while True:
            try:
                stack_update = self.connection.describe_stacks(stack_name_or_id=stack)
                resources = self.connection.list_stack_resources(stack)
            except boto.exception.BotoServerError as e:
                print colored("Error: ", "red") + e.message
                exit(1)
            os.system('cls' if os.name == 'nt' else 'clear')

            su = stack_update[0]
            print "Name:    %s" % su.stack_name
            print "Created: %s" % su.creation_time
            print "Status:  %s" % self._colorize_status(su.stack_status)
            print
            print colored(" %-23s %-35s %-18s " % ("Resource", "Type", "Status"), 'white', 'on_blue')
            for r in resources:
                print " %-23s %-35s %-27s " % (r.logical_resource_id[0:23],
                                               r.resource_type[0:35],
                                               self._colorize_status(r.resource_status)[0:27])
            if su.stack_status == "CREATE_COMPLETE" or su.stack_status == "UPDATE_COMPLETE":
                wrapper = TextWrapper(width=54)
                print
                print colored(" %-23s %-54s " % ("Output", "Value"), 'grey', 'on_yellow')
                for o in su.outputs:
                    v = wrapper.wrap(o.value)
                    print " %-23s %-54s " % (o.key[0:24], v.pop(0))
                    if len(v) > 0:
                        print "                        ", "\n                         ".join(v)
            print "\nMore information: https://console.aws.amazon.com/cloudformation/home"
            if su.stack_status.endswith("FAILED") or su.stack_status.endswith("COMPLETE"):
                break
            else:
                print "Updating in  ",
                for i in xrange(9, -1, -1):
                    sys.stdout.write('\b')
                    sys.stdout.write(str(i))
                    sys.stdout.flush()
                    time.sleep(1)
                print

    def _colorize_status(self, status):
        if status.endswith("COMPLETE"):
            return colored(status, "green")
        elif status.endswith("IN_PROGRESS"):
            return colored(status, "yellow")
        elif status.endswith("FAILED"):
            return colored(status, "red")
        else:
            return status


def parse_cf_json(str):
    try:
        cf_dict = json.loads(str, object_pairs_hook=collections.OrderedDict)
    except ValueError as e:
        print "Error loading JSON template file."
        print e
        exit(1)
    else:
        return cf_dict


def get_cf_json(order_dict, pretty=False):
    outstr = None
    if pretty:
        outstr = json.dumps(order_dict, indent=2, separators=(',', ': '))
    else:
        outstr = json.dumps(order_dict)
    return outstr



#!/usr/bin/env python

from setuptools import setup
import os

# Markdown to ReStructuredText conversion
long_description = 'Nepho is a command-line tool that orchestrates the creation of complete working application stacks on virtual infrastructure. Initially targeting Amazon Web Services as well as Vagrant, Nepho abstracts datacenter creation, instance configuration, and application deployment into portable "cloudlets" that can be shared between developers and teams.'
if os.path.exists('README.rst'):
    long_description = open('README.rst').read()


setup(
    name         = 'nepho',
    version      = '1.2.1',
    url          = 'http://github.com/huit/nepho',
    description  = 'Simplified cloud orchestration tool for constructing virtual data centers',
    long_description = long_description,
    packages     = ['nepho', 'nepho.core', 'nepho.cli', 'nepho.providers'],
    author       = 'Harvard University Information Technology',
    author_email = 'ithelp@harvard.edu',
    license      = 'LICENSE.txt',
    scripts      = ['bin/nepho_completer'],
    entry_points = {
        'console_scripts': [
            'nepho=nepho.cli.bootstrap:run',
        ]
    },
    dependency_links = [
        'git+git://github.com/cement/cement.git@2.1.4.dev20131029203905#egg=cement-2.1.4-dev'
    ],
    install_requires = [
        'boto>=2.0',
        'Jinja2',
        'PyYAML',
        'cement>=2.1,==2.1.4-dev',
        'termcolor',
        'colorama',
        'gitpython==0.3.2.RC1',
        'requests>=1.2.3',
        'python-vagrant==0.4.0'
    ],
    setup_requires = [
        'setuptools_git>=1.0',
        'pypandoc>=0.7.0',
    ],
    test_suite = 'nose.collector',
    tests_require = [
        'flake8>=2.1.0',
        'nose>=1.3.0',
        'coverage>=3.7',
    ],
)



import yaml
from os.path import basename, join, isfile
from nepho.core.common import merge


def load_and_merge_blueprint(paths):
    """
    Take multiple directory paths for the same blueprint, find and read/validate
    each YAML file, and merge them, with later paths taking precedence. Return
    result as a dict.
    """
    blueprint = None
    for p in paths:
        blueprint_key = basename(p)
        blueprint_file = join(p, "%s.yaml") % (blueprint_key)
        if isfile(blueprint_file):
            try:
                f = open(blueprint_file)
                blueprint_yaml = yaml.safe_load(f)
                f.close()
            except IOError as e:
                print "Error loading or parsing deployment blueprint file \"%s\"" % (blueprint_file)
                print e
                exit(1)
            if blueprint is not None:
                blueprint = merge(blueprint_yaml, blueprint)
            else:
                blueprint = blueprint_yaml
    return blueprint



# coding: utf-8
import os
import sys
import tempfile
from time import time
import re
import yaml
import requests
import glob
from termcolor import colored
from shutil import rmtree, copyfileobj

from git import Repo

from nepho.core import common, blueprint


class Cloudlet:
    """A class that encompasses a cloudlet"""

    def __init__(self, name, cloudlet_path=None, url=None):
        self.path = cloudlet_path
        self.url = url
        self.name = name

        # If specified initialize this from a remote repo
        if url is not None:
            self.clone(url)

        # now load definition, and fail is unable
        self.definition = None
        if self.path is not None:
            try:
                self.definition = yaml.safe_load(open(os.path.join(self.path, "cloudlet.yaml")))
            except Exception:
                print "Error loading cloudlet YAML file!"
                exit(1)

        # include metadata into the definition dictionary
        self.definition['name'] = self.name
        self.definition['url'] = self.url
        self.definition['path'] = self.path

    def serialize(self):
        """Returns the cloudlets data definition as a JSON string."""

    def blueprint(self, name):
        """Return a blueprint by name."""
        bps = self.blueprints()
        for bp in bps:
            if bp.name == name:
                return bp

    def blueprints(self):
        """Returns a list of blueprints."""
        blueprint_dir = os.path.join(self.path, "blueprints")
        blueprint_files = list()
        if os.path.isdir(blueprint_dir):
            blueprint_files.extend(glob.glob(os.path.join(blueprint_dir, '*.yaml')))
        else:
            return None

        blueprints = list()
        for f in blueprint_files:
            blueprints.append(blueprint.Blueprint(self, f))

        return blueprints

    def clone(self, url):
        """
        Creates a local cloudlet as a git repo on disk
         from the supplied remote git URL.
        """
        try:
            temp_repo = tempfile.mkdtemp()
            validate = Repo.init(temp_repo, bare=True)
            validate.git.ls_remote(url, heads=True)
        except Exception as e:
            print colored("Error: ", "red") + "Invalid or inaccessible remote repository URL.\n"
            print e
            exit(1)
        else:
            try:
                repo = Repo.clone_from(url, self.path)
                repo.submodule_update(init=True)
            except Exception as e:
                print "Cloudlet install failed."
                print e
                exit(1)
            else:
                print "Cloudlet installed: %s" % (self.path)
        finally:
            rmtree(temp_repo)

    def update(self):
        """Update the local cloudlet git repo on disk from any origin."""

        print "Updating cloudlet: %s" % (self.path)
        repo = Repo(self.path)
        repo.remotes.origin.pull()
        repo.submodule_update()

    def publish(self):
        """Update the remote cloudlet git repo from the local one."""

        print "Publishing cloudlet: %s" % (self.path)
        repo = Repo(self.path)
        repo.remotes.origin.push()

    def archive(self, repo_name, archive_dir=tempfile.gettempdir()):
        """Archives the cloudlet on disk as a tar file, and removes it."""
        repo = Repo(self.path)
        try:
            print "Archiving %s to %s." % (repo_name, archive_dir)
            archive_file = os.path.join(archive_dir, "%s.tar") % (repo_name)

            # TODO: If a tar already exists, rotate/increment it. I tried using
            # logging.handlers.RotatingFileHandler for this, but it didn't quite
            # work (gave zero-length files).

            # Archive and delete the repository
            repo.archive(open(archive_file, "w"))
        except Exception as e:
            print "Archive failed -- aborting!"
            print e
            exit(1)
        else:
            if os.path.islink(self.path):
                os.unlink(self.path)
            else:
                rmtree(self.path)

    def uninstall(self):
        """Meant to remove the cloudlet from disk. For now a no-op."""
        # No-op here ... done by archive above for now
        pass

    def get_path(self):
        """Return the  path to the cloudlet's root dir."""
        return self.path


class CloudletManager:
    """A class to create, lookup, and manage cloudlets"""

    def __init__(self, app_obj):
        self.cloudlet_dirs  = app_obj.cloudlet_dirs
        self.registry       = app_obj.config.get('nepho', 'cloudlet_registry_url')
        self.cache_dir      = app_obj.config.get('nepho', 'cache_dir')
        self.registry_cache = os.path.join(self.cache_dir, "registry.yaml")
        self.update_registry()

    def all_cloudlet_dirs(self):
        """Returns a list of paths to directories that contain cloudlets on disk."""
        return self.cloudlet_dirs

    def all_cloudlet_paths(self):
        """Returns a list of paths to cloudlets on disk."""
        cloudlet_paths = list()
        for dir in self.cloudlet_dirs:
            cloudlet_paths.extend(glob.glob(os.path.join(dir, '*')))
        return cloudlet_paths

    def new(self, name, target_dir=None, url=None):
        """Create a new Cloudlet of a given name within a selected cloudlets dir, cloning from a URL."""
        if target_dir is None:
            target_dir = self.all_cloudlet_dirs()[0]

        cloudlet_path = os.path.join(target_dir, name)
        return Cloudlet(name, cloudlet_path, url)

    def find(self, name=None, multiple=False):
        """Search cloudlet locations for a cloudlet matching the given name, or all if name is None."""

        cloudlet_paths = self.all_cloudlet_paths()

        if name is None:
            paths = [path for path in cloudlet_paths]
        else:
            paths = [path for path in cloudlet_paths if (os.path.split(path))[1] == name]
        cloudlets = []
        for p in paths:
            cloudlets.append(Cloudlet(name, p))
        if len(cloudlets) == 0:
            return None
        if multiple is True:
            return cloudlets
        else:
            return cloudlets[0]

    def find_cloudlet_path(self, name, multiple=False):
        """Returns a list of paths to cloudlets based on the given name."""

        cloudlets = self.find(None, multiple)
        paths = [c.path for c in cloudlets]
        if multiple is True:
            return paths
        else:
            return paths[0]

    def list(self):
        """Returns a list of all cloudlets."""

        cloudlts = self.find(None, True)
        return cloudlts

    def clear_registry(self):
        """Removes any cached registry info in the local nepho directory."""
        try:
            os.remove(self.registry_cache)
        except Exception:
            pass

    def update_registry(self):
        """Pulls down a fresh copy of the cloudlet registry."""

        # If the local registry is missing, empty, or stale (over 4 hours old)
        # update it from the configured URL. In either case, return the YAML object
        if not os.path.exists(self.registry_cache) or os.path.getsize(self.registry_cache) == 0 or (time() - os.path.getmtime(self.registry_cache)) > 14400:
            print "Updating cloudlet registry from: %s" % (self.registry)
            try:
                response = requests.get(self.registry, stream=True)
                with open(self.registry_cache, 'wb') as out_file:
                    yaml.safe_dump(yaml.safe_load(response.raw), out_file, default_flow_style=True)
                    del response
            except yaml.YAMLError, detail:
                print "Invalid registry data received: ", detail
                pass
            except IOError:
                print "Error updating cloudlet registry. Using last cached version.\n"
                pass

            with open(self.registry_cache, 'r') as yaml_file:
                try:
                    return yaml.safe_load(yaml_file)
                except yaml.YAMLError, detail:
                    print "Invalid registry data found: ", detail
                    print "Try running the nepho command again.\n"
                    os.unlink(self.registry_cache)
                    sys.exit(1)
        else:
            with open(self.registry_cache, 'r') as yaml_file:
                try:
                    return yaml.safe_load(yaml_file)
                except yaml.YAMLError, detail:
                    print "Invalid registry data found: ", detail
                    print "Try running the nepho command again.\n"
                    os.unlink(self.registry_cache)
                    sys.exit(1)

    def get_registry(self):
        """Returns a dictionary with registry values."""
        self.update_registry()
        with open(self.registry_cache, 'r') as yaml_file:
            return yaml.safe_load(yaml_file)
